<?php
namespace Misuzu;

use RuntimeException;
use Misuzu\Users\User;

if(!User::hasCurrent() || !perms_check_user(MSZ_PERMS_USER, User::getCurrent()->getId(), MSZ_PERM_USER_MANAGE_BANS)) {
    echo render_error(403);
    return;
}

$userInfos = [
    (string)User::getCurrent()->getId() => User::getCurrent(),
];

$filterUser = null;
if(filter_has_var(INPUT_GET, 'u')) {
    $filterUserId = (int)filter_input(INPUT_GET, 'u', FILTER_SANITIZE_NUMBER_INT);
    try {
        $filterUser = User::byId($filterUserId);
        $userInfos[(string)$filterUser->getId()] = $filterUser;
    } catch(RuntimeException $ex) {
        echo render_error(404);
        return;
    }
}

$bans = $msz->getBans();
$pagination = new Pagination($bans->countBans(userInfo: $filterUser), 10);

if(!$pagination->hasValidOffset()) {
    echo render_error(404);
    return;
}

$banList = [];
$banInfos = $bans->getBans(userInfo: $filterUser, activeFirst: true, pagination: $pagination);

foreach($banInfos as $banInfo) {
    if(array_key_exists($banInfo->getUserId(), $userInfos))
        $userInfo = $userInfos[$banInfo->getUserId()];
    else
        $userInfos[$banInfo->getUserId()] = $userInfo = User::byId((int)$banInfo->getUserId());

    if(!$banInfo->hasModId())
        $modInfo = null;
    elseif(array_key_exists($banInfo->getModId(), $userInfos))
        $modInfo = $userInfos[$banInfo->getModId()];
    else
        $userInfos[$banInfo->getModId()] = $modInfo = User::byId((int)$banInfo->getModId());

    $banList[] = [
        'info' => $banInfo,
        'user' => $userInfo,
        'mod' => $modInfo,
    ];
}

Template::render('manage.users.bans', [
    'manage_bans' => $banList,
    'manage_bans_pagination' => $pagination,
    'manage_bans_filter_user' => $filterUser,
]);
