<?php
namespace Misuzu;

use RuntimeException;
use Misuzu\Users\User;

if(!User::hasCurrent() || !perms_check_user(MSZ_PERMS_USER, User::getCurrent()->getId(), MSZ_PERM_USER_MANAGE_NOTES)) {
    echo render_error(403);
    return;
}

$userInfos = [
    (string)User::getCurrent()->getId() => User::getCurrent(),
];

$filterUser = null;
if(filter_has_var(INPUT_GET, 'u')) {
    $filterUserId = (int)filter_input(INPUT_GET, 'u', FILTER_SANITIZE_NUMBER_INT);
    try {
        $filterUser = User::byId($filterUserId);
        $userInfos[(string)$filterUser->getId()] = $filterUser;
    } catch(RuntimeException $ex) {
        echo render_error(404);
        return;
    }
}

$modNotes = $msz->getModNotes();
$pagination = new Pagination($modNotes->countNotes(userInfo: $filterUser), 10);

if(!$pagination->hasValidOffset()) {
    echo render_error(404);
    return;
}

$notes = [];
$noteInfos = $modNotes->getNotes(userInfo: $filterUser, pagination: $pagination);

foreach($noteInfos as $noteInfo) {
    if(array_key_exists($noteInfo->getUserId(), $userInfos))
        $userInfo = $userInfos[$noteInfo->getUserId()];
    else
        $userInfos[$noteInfo->getUserId()] = $userInfo = User::byId((int)$noteInfo->getUserId());

    if(!$noteInfo->hasAuthorId())
        $authorInfo = null;
    elseif(array_key_exists($noteInfo->getAuthorId(), $userInfos))
        $authorInfo = $userInfos[$noteInfo->getAuthorId()];
    else
        $userInfos[$noteInfo->getAuthorId()] = $authorInfo = User::byId((int)$noteInfo->getAuthorId());

    $notes[] = [
        'info' => $noteInfo,
        'user' => $userInfo,
        'author' => $authorInfo,
    ];
}

Template::render('manage.users.notes', [
    'manage_notes' => $notes,
    'manage_notes_pagination' => $pagination,
    'manage_notes_filter_user' => $filterUser,
]);
