<?php
namespace Misuzu;

use RuntimeException;
use Misuzu\Users\User;

if(!User::hasCurrent() || !perms_check_user(MSZ_PERMS_USER, User::getCurrent()->getId(), MSZ_PERM_USER_MANAGE_WARNINGS)) {
    echo render_error(403);
    return;
}

$userInfos = [
    (string)User::getCurrent()->getId() => User::getCurrent(),
];

$filterUser = null;
if(filter_has_var(INPUT_GET, 'u')) {
    $filterUserId = (int)filter_input(INPUT_GET, 'u', FILTER_SANITIZE_NUMBER_INT);
    try {
        $filterUser = User::byId($filterUserId);
        $userInfos[(string)$filterUser->getId()] = $filterUser;
    } catch(RuntimeException $ex) {
        echo render_error(404);
        return;
    }
}

$warns = $msz->getWarnings();
$pagination = new Pagination($warns->countWarnings(userInfo: $filterUser), 10);

if(!$pagination->hasValidOffset()) {
    echo render_error(404);
    return;
}

$warnList = [];
$warnInfos = $warns->getWarnings(userInfo: $filterUser, pagination: $pagination);

foreach($warnInfos as $warnInfo) {
    if(array_key_exists($warnInfo->getUserId(), $userInfos))
        $userInfo = $userInfos[$warnInfo->getUserId()];
    else
        $userInfos[$warnInfo->getUserId()] = $userInfo = User::byId((int)$warnInfo->getUserId());

    if(!$warnInfo->hasModId())
        $modInfo = null;
    elseif(array_key_exists($warnInfo->getModId(), $userInfos))
        $modInfo = $userInfos[$warnInfo->getModId()];
    else
        $userInfos[$warnInfo->getModId()] = $modInfo = User::byId((int)$warnInfo->getModId());

    $warnList[] = [
        'info' => $warnInfo,
        'user' => $userInfo,
        'mod' => $modInfo,
    ];
}

Template::render('manage.users.warnings', [
    'manage_warns' => $warnList,
    'manage_warns_pagination' => $pagination,
    'manage_warns_filter_user' => $filterUser,
]);
